Module: ElasticsearchRecord

Extended by:
ActiveSupport::Autoload
Defined in:
lib/elasticsearch_record/relation/query_clause.rb,
lib/elasticsearch_record.rb,
lib/elasticsearch_record/base.rb,
lib/elasticsearch_record/core.rb,
lib/elasticsearch_record/query.rb,
lib/elasticsearch_record/errors.rb,
lib/elasticsearch_record/result.rb,
lib/elasticsearch_record/version.rb,
lib/elasticsearch_record/querying.rb,
lib/elasticsearch_record/model_api.rb,
lib/elasticsearch_record/gem_version.rb,
lib/elasticsearch_record/persistence.rb,
lib/elasticsearch_record/model_schema.rb,
lib/elasticsearch_record/instrumentation.rb,
lib/elasticsearch_record/statement_cache.rb,
lib/elasticsearch_record/schema_migration.rb,
lib/elasticsearch_record/extensions/relation.rb,
lib/elasticsearch_record/relation/core_methods.rb,
lib/elasticsearch_record/relation/query_methods.rb,
lib/elasticsearch_record/relation/value_methods.rb,
lib/elasticsearch_record/instrumentation/railtie.rb,
lib/elasticsearch_record/relation/result_methods.rb,
lib/elasticsearch_record/relation/query_clause_tree.rb,
lib/elasticsearch_record/relation/calculation_methods.rb,
lib/elasticsearch_record/instrumentation/log_subscriber.rb,
lib/elasticsearch_record/instrumentation/controller_runtime.rb,
lib/elasticsearch_record/tasks/elasticsearch_database_tasks.rb,
sig/elasticsearch_record.rbs

Overview

:nodoc:

Defined Under Namespace

Modules: Core, Extensions, Instrumentation, ModelSchema, Persistence, Querying, Relation, Tasks Classes: Base, ElasticsearchRecordError, ModelApi, Query, ResponseResultError, Result, SchemaMigration, StatementCache

Constant Summary collapse

VERSION =

Returns:

  • (String)

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Attribute Details

#decorate_table_namesObject

:singleton-method: Specifies if the table (index) statements resolve their provided table name(s) with the table_name_prefix & table_name_suffix of the connection config. As default every statement decorates, so a migration only ever names the base table (index). Disabling this flag restores the former, opt-in behaviour, where the decoration had to be applied by hand through #_env_table_name.

HINT: this only provides the DEFAULT for a statement that was not given an explicit decorate: argument - a single call can always opt in or out on its own.

see @ ActiveRecord::ConnectionAdapters::Elasticsearch::TableStatements



81
82
83
# File 'lib/elasticsearch_record.rb', line 81

def decorate_table_names
  @decorate_table_names
end

#error_on_transactionObject

:singleton-method: Specifies if a exception should be raised while using transactions. Since ActiveRecord does not have any configuration option to support transactions and Elasticsearch does NOT support transactions, it may be risky to ignore them. As default, transactional are 'silently swallowed' to not break any existing applications... However enabling this flag will surely fail transactional tests ...



66
67
68
# File 'lib/elasticsearch_record.rb', line 66

def error_on_transaction
  @error_on_transaction
end

Class Method Details

.gem_versionObject

Returns the version of the currently loaded module as a Gem::Version



5
6
7
# File 'lib/elasticsearch_record/gem_version.rb', line 5

def self.gem_version
  Gem::Version.new VERSION::STRING
end

.versionObject

Returns the version of the currently loaded Gem as a Gem::Version



7
8
9
# File 'lib/elasticsearch_record/version.rb', line 7

def self.version
  gem_version
end