Class: ElasticsearchRecord::InternalMetadata

Inherits:
ActiveRecord::InternalMetadata
  • Object
show all
Defined in:
lib/elasticsearch_record/internal_metadata.rb

Instance Method Summary collapse

Instance Method Details

#enabled?Boolean

overwrite method to always disable the internal metadata index.

Elasticsearch cannot serve the internal metadata table (see ElasticsearchAdapter#use_metadata_table?) - but since rails 7.2 the flag is resolved through the database config (default: enabled) instead of the adapter, so a default configuration would suddenly create & write a 'ar_internal_metadata' index on the cluster.

Returns:

  • (Boolean)


15
16
17
# File 'lib/elasticsearch_record/internal_metadata.rb', line 15

def enabled?
  false
end